'use strict';

function _interopRequireDefault(obj) { return obj && obj.__esModule ? obj : { 'default': obj }; }

var _react = require('react');

var _react2 = _interopRequireDefault(_react);

var _reactDom = require('react-dom');

var _reactDom2 = _interopRequireDefault(_reactDom);

/**
 * Overlay Mixin
 *
 * @desc `overlay` is something like Popover, Modal, etc.
 * */

module.exports = {

    //propTypes: {
    //    container: React.PropTypes.node
    //},

    componentDidMount: function componentDidMount() {
        this._renderOverlay();
    },

    componentDidUpdate: function componentDidUpdate() {
        this._renderOverlay();
    },

    // Remove Overlay related DOM node
    componentWillUnmount: function componentWillUnmount() {
        this._unmountOverlay();

        if (this._overlayWrapper) {
            this.getContainerDOMNode().removeChild(this._overlayWrapper);
            this._overlayWrapper = null;
        }
    },

    // Create Overlay wrapper
    _mountOverlayWrapper: function _mountOverlayWrapper() {
        this._overlayWrapper = document.createElement('div');
        this.getContainerDOMNode().appendChild(this._overlayWrapper);
    },

    // Render Overlay to wrapper
    _renderOverlay: function _renderOverlay() {
        if (!this._overlayWrapper) {
            this._mountOverlayWrapper();
        }

        var overlay = this.renderOverlay();

        if (overlay !== null) {
            this._overlayInstance = _reactDom2['default'].render(overlay, this._overlayWrapper);
        } else {
            // Unmount if the component is null for transitions to null
            this._unmountOverlay();
        }
    },

    // Remove a mounted Overlay from wrapper
    _unmountOverlay: function _unmountOverlay() {
        _react2['default'].unmountComponentAtNode(this._overlayWrapper);
        this._overlayInstance = null;
    },

    getOverlayDOMNode: function getOverlayDOMNode() {

        if (this._overlayInstance) {
            return _reactDom2['default'].findDOMNode(this._overlayInstance);
        }

        return null;
    },

    getContainerDOMNode: function getContainerDOMNode() {
        return _reactDom2['default'].findDOMNode(this.props.container) || document.body;
    }
};